/**
 * @module ctt
 * @submodule tree-view
 */
(function() {
var $ = dixim.ctt.jQuery;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;

var TreeView;
var prototype;

var NODE_ID_PREFIX = 'ctt-tree-view-';
var NODE_CLASS = 'ctt-tree-view';
var NODE_CLASS_SELECTED = 'ctt-tree-view-selected';
var TEMPLATE_TREE_ROOT = '<ul class="filetree"></ul>';
var TEMPLATE_CONTAINER_NODE = '<li class="closed"><span class="folder" id="${id}">${label}</span><ul></ul></li>';
var TEMPLATE_LEAF_NODE = '<li><span class="file" id="${id}">${label}</span></li>';
var TEMPLATE_LEAF_ERROR_NODE = '<li><span class="error" id="${id}">${label}</span></li>';

/**
 * @class TreeView
 * @constructor
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {String} configs.headerNode
 */
TreeView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(TreeView, EventTarget);
dixim.ctt.TreeView = TreeView;
prototype = TreeView.prototype;

/**
 * @event clickNode
 */
var CLICK_NODE = 'clickNode';

/**
 * @event clickHeader
 */
var CLICK_HEADER = 'clickHeader';

/**
 * @event toggle
 */
var TOGGLE = 'toggle';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {String} configs.headerNode
 */
TreeView.prototype._initialize = function(configs) {
  var children;

  this._containerNode = configs.containerNode;
  this._headerNode = configs.headerNode;
  this._container = $(configs.containerNode).addClass(NODE_CLASS);
  this._delegate = configs.delegate;
  this._treeView = null;

  // TODO: consider data structure.
  children = configs.children || [];
  this._records = children;
};

TreeView.prototype._encodeToNodeId = function(id) {
  id = dixim.xml.encodeId(id);
  return NODE_ID_PREFIX + id;
};

TreeView.prototype._decodeToId = function(nodeId) {
  nodeId = nodeId.replace(new RegExp('^' + NODE_ID_PREFIX), '');
  return dixim.xml.decodeId(nodeId);
};

TreeView.prototype._getRecord = function(id) {
  var i;
  var len;
  var records;

  records = this._records;
  len = records.length;
  for (i = 0; i < len; i += 1) {
    if (records[i].id === id) {
      return records[i];
    }
  }
  return null;
};

TreeView.prototype._didClickHeader = function(e) {
  this.dispatchEvent(new Event(CLICK_HEADER, {treeView: this}));
};

TreeView.prototype._didClickNode = function(e) {
  var nodeId;
  var id;
  var node;
  var event;
  var args = {};

  node = $(e.target);
  if (!node.is("span") || (!node.hasClass('folder') && !node.hasClass('file'))) {
    return;
  }

  nodeId = $(node).attr('id');
  id = this._decodeToId(nodeId);

  args.record = this._getRecord(id);
  args.treeView = this;

  event = new Event(CLICK_NODE, args);
  this.dispatchEvent(event);
};

TreeView.prototype._didToggle = function() {
  var node;
  var id;
  var event;
  var args;
  var opened = false;

  node = $(arguments[0]);
  if (!(opened = node.hasClass('collapsable'))) {
    $('ul', node).empty();
  }

  id = this._decodeToId($('span', node).attr('id'));
  args = {};
  args.record = this._getRecord(id);
  args.opened = opened;
  args.treeView = this;

  event = new Event(TOGGLE, args);
  this.dispatchEvent(event);
};

TreeView.prototype._createTreeView = function() {
  var that = this;
  var tv;
  var toggle;

  toggle = function() {
    var args = [this].concat(arguments);
    that._didToggle.apply(that, args);
  };

  if (!!this._headerNode) {
    $(this._headerNode).
      appendTo(this._container).
      addClass(NODE_CLASS + '-header').
      bind('click.TreeView', $.proxy(this._didClickHeader, this));
  }

  tv = $(TEMPLATE_TREE_ROOT).hide().appendTo(this._container);
  tv = tv.treeview({toggle: toggle}).bind('click.TreeView', $.proxy(this._didClickNode, this));
  this._treeView = tv;
  this._addChildElements(this._records);
};

/**
 * Adds child nodes to the node specified.
 *
 * @param {Object[]} children
 *  Children to be added to the node.
 *  Each object has the following properties.
 *    <dl>
 *      <dt>{String} id</dt>
 *        <dd>an ID of the object. The ID must be unique.</dd>
 *      <dt>{String} [label]</dt>
 *        <dd>a label of the node for the object.</dd>
 *    </dl>
 * @param {String} [parent] A parent ID for the children.
 * @remark If the parent is not found, no children are added.
 */
TreeView.prototype.addChildren = function(children, parent) {
  var i, len;
  var list = [];

  len = children.length;
  for (i = 0; i < len; i += 1) {
    list.push($.extend(true, {parent: parent}, children[i]));
  }
  this._addChildElements(children, parent);
  this._records = this._records.concat(list);
};

/**
 * @method _addChildElements
 * @private
 */
TreeView.prototype._addChildElements = function(children, parent, configs) {
  var tv = this._treeView;
  var parentNode = null;
  var child;
  var i;
  var len;
  var tmpEl;
  var childrenEl;
  var html;
  var template;

  if (typeof parent === 'undefined') {
    parentNode = tv;
  } else {
    parent = $('#' + this._encodeToNodeId(parent), tv);
    parentNode = $('+ ul', parent);
    if (!parentNode || !parentNode.length) {
      return;
    }
  }

  len = children.length;
  tmpEl = $('<div/>');
  for (i = 0; i < len; i += 1) {
    child = children[i];
    if (child.type === 'leaf') {
      template = TEMPLATE_LEAF_NODE;
    } else if (child.type === 'error') {
      template = TEMPLATE_LEAF_ERROR_NODE;
    } else {
      template = TEMPLATE_CONTAINER_NODE;
    }
    html = template.
           replace('${label}', dixim.xml.escape(child.label)).
           replace('${id}', this._encodeToNodeId(child.id));
    tmpEl.append($(html));
  }

  childrenEl = tmpEl.children();
  parentNode.append(childrenEl).treeview({add: childrenEl});
};

/**
 * @method removeChildren
 * @param {String} parentId
 */
TreeView.prototype.removeChildren = function(parentId) {
  var parent;
  var tv = this._treeView;

  parent = $(['#', this._encodeToNodeId(parentId)].join(''), tv);
  parent = $('+ ul', parent);
  parent.empty();
};

TreeView.prototype.clear = function() {
  this._treeView.empty();
};

TreeView.prototype.show = function() {
  if (!!!this._treeView) {
    this._createTreeView();
  }
  this._treeView.show();
};

TreeView.prototype.hide = function() {
  this._treeView.hide();
};

TreeView.prototype.destroy = function() {
  if (!!!this._treeView) return;
  this._treeView.unbind('.TreeView');
  this._treeView.remove();
  this._treeView = null;
};

TreeView.prototype._clearSelection = function() {
  $(this._treeView).find('.' + NODE_CLASS_SELECTED).removeClass(NODE_CLASS_SELECTED);
};

TreeView.prototype.clearSelection = function() {
  this._clearSelection();
};

TreeView.prototype.selectNode = function(record) {
  var nodeId;
  var tv = this._treeView;

  this._clearSelection();
  nodeId = this._encodeToNodeId(record.id);
  $(['#', nodeId].join(''), tv).addClass(NODE_CLASS_SELECTED);
};

}());
