/**
 * @module ctt
 * @submodule dms-listing-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var DmsListingView = dixim.ctt.DmsListingView;

var DmsListingController;
var prototype;

/**
 * @class DmsListingController
 * @constructor
 * @param {Object} configs
 */
DmsListingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(DmsListingController, EventTarget);
dixim.ctt.DmsListingController = DmsListingController;
prototype = DmsListingController.prototype;

/**
 * Triggered when the selection is changed.
 * 
 * @event changeSelection
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.record
 */
var CHANGE_SELECTION = 'changeSelection';

/**
 * Triggered when the list has been updated.
 * 
 * @event updateList
 */
var UPDATE_LIST = 'updateList';

/**
 * Triggered when the list updating was failed.
 * 
 * @event updateListError
 */
var UPDATE_LIST_ERROR = 'updateListError';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  this._containerNode = configs.containerNode;
  this._model = configs.model;
  this._filter = configs.filter || '';
  this._view = new DmsListingView({containerNode: this._containerNode});
  this._session = null;
  this._records = [];
};

/**
 * @method start
 */
prototype.start = function() {
  this._view.addEventListener('render', this._didRenderView, this);
  this._view.addEventListener('clickItem', this._didClickItem, this);
  this._view.addEventListener('changeSelection', this._didChangeSelection, this);
  this._model.addEventListener('dmsList', this._didFetchDmsList, this);
  this._model.addEventListener('dmsListError', this._didFailFetchDmsList, this);
  this._view.show();
};

prototype._didRenderView = function(e) {
  this._fetchDmsList();
};

prototype._fetchDmsList = function() {
  this._session = this._model.dmsList();
};

prototype._didClickItem = function(e) {
  this._view.selectRow(e.args.record);
};

prototype._didChangeSelection = function(e) {
  this.dispatchEvent(new Event(CHANGE_SELECTION));
};

var doesDmsSupportProfiles = function(dms, profiles) {
  var i;
  var len;

  if (!!!dms.supportedProfiles) return false;

  len = profiles.length;
  for (i = 0; i < len; i += 1) {
    if (!dms.supportedProfiles.match(new RegExp('(^|,)' + profiles[i] + '(,|$)'))) {
      return false;
    }
  }
  return true;
};

prototype._filterDmsList = function(list) {
  var ret = [];
  var i;
  var len;
  var profiles;

  profiles = this._filter.split(',');
  if (!profiles.length) return list.slice(0);

  len = list.length;
  for (i = 0; i < len; i += 1) {
    if (!doesDmsSupportProfiles(list[i], profiles)) continue;
    ret.push(list[i]);
  }
  return ret;
};

prototype._didFetchDmsList = function(e) {
  var list;
  var session = e.args.session;

  if (this._session !== session) return;

  this._session = null;
  if (session.isCancelled()) return;

  list = this._filterDmsList(e.args.data.results);
  this._view.removeAllRows();
  this._view.addRows(list);
  this._records = list;
  this.dispatchEvent(new Event(UPDATE_LIST));
};

prototype._didFailFetchDmsList = function(e) {
  var session = e.args.session;

  if (this._session !== session) return;
  this._records = [];
  this._session = null;
  this._view.removeAllRows();
  this.dispatchEvent(new Event(UPDATE_LIST_ERROR));
};

prototype.getSelectedDms = function() {
  return this._getSelectedDms();
};

prototype._getSelectedDms = function() {
  var list;

  list = this._view.getSelectedRows();
  return list.length ? list[0] : null;
};

prototype.getSelectedDmsElement = function() {
  var row;
  var el;

  row = this._getSelectedDms();
  if (!!!row) return null;

  el = this._view.getRowElement(row);
  return el;
};

prototype.stop = function() {
  if (!!!this._view) return;
  this._view.hide();
};

prototype.destroy = function() {
  if (!!!this._view) return;
  if (!!this._session) {
    this._session.cancel();
    this._session = null;
  }
  this._model.removeEventListener('dmsList', this._didFetchDmsList, this);
  this._model.removeEventListener('dmsListError', this._didFailFetchDmsList, this);
  this._view.destroy();
};

prototype.reload = function() {
  this._fetchDmsList();
};

prototype.getNumberOfDmss = function() {
  return this._records.length;
};

prototype.selectDms = function(dms) {
  var records = this._records;
  var i;
  var len = records.length;

  for (i = 0; i < len; i += 1) {
    if (dms.udn === records[i].udn) {
      this._selectDms(records[i]);
      break;
    }
  }
};

prototype._selectDms = function(dms) {
  this._view.selectRow(dms);
};

prototype.selectFirstDms = function() {
  var records = this._records;

  if (records.length) {
    this._selectDms(records[0]);
  }
};

}());
